#!/usr/bin/python3
"""SAI generator for PA Validation"""

import os
import sys

from dpugen.confbase import (
    ConfBase,
    ipa
)
from dpugen.confutils import common_main


class PaValidation(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):
            vtep_remote = str(ipa(p.PAR) + int(ipa(p.IP_STEP1)) * eni_index)

            self.num_yields += 1
            yield {
                'name': f'pa_validation_#eni{eni}',
                'op': 'create',
                'type': 'SAI_OBJECT_TYPE_PA_VALIDATION_ENTRY',
                'key': {
                    'switch_id': '$SWITCH_ID',
                    'sip': vtep_remote,
                    'vnet_id': f'$vnet_#eni{eni}'
                },
                'attributes': [
                    'SAI_PA_VALIDATION_ENTRY_ATTR_ACTION', 'SAI_PA_VALIDATION_ENTRY_ACTION_PERMIT'
                ]
            }


if __name__ == '__main__':
    conf = PaValidation()
    common_main(conf)



#!/usr/bin/python3
"""SAI generator for Outbound Routing"""

import math
import os
import sys

from dpugen.confbase import (
    ConfBase,
    ipa
)
from dpugen.confutils import common_main


class OutboundRouting(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        nr_of_routes_prefixes = int(math.log(p.IP_ROUTE_DIVIDER_PER_ACL_RULE, 2))
        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):
            print('     route:eni:%d' % eni, file=sys.stderr)
            vtep_eni = str(ipa(p.PAL) + int(ipa(p.IP_STEP1)) * eni_index)
            added_route_count = 0
            for nsg_index in range(1, (p.ACL_NSG_COUNT*2+1)):
                for acl_index in range(1, (p.ACL_RULES_NSG+1)):
                    ip_map_count = 0
                    remote_ip = str(ipa(p.IP_R_START) + eni_index * int(ipa(p.IP_STEP_ENI)) + (nsg_index - 1)
                                    * int(ipa(p.IP_STEP_NSG)) + (acl_index - 1) * int(ipa(p.IP_STEP_ACL)))
                    no_of_route_groups = p.IP_PER_ACL_RULE * 2 // p.IP_ROUTE_DIVIDER_PER_ACL_RULE
                    # for ip_index in range(0, no_of_route_groups + 1):
                    for ip_index in range(0, no_of_route_groups):
                        ip_prefix = str(ipa(remote_ip) - 1 + ip_index *
                                        p.IP_ROUTE_DIVIDER_PER_ACL_RULE * int(ipa(p.IP_STEP1)))
                        for prefix_index in range(nr_of_routes_prefixes, 0, -1):
                            nr_of_ips = 1 << (prefix_index-1)
                            mask = 32 - prefix_index + 1
                            if mask == 32:
                                ip_prefix = str(ipa(ip_prefix) + 1)

                            if (eni % 4) == 1:
                                # routes that have a mac mapping
                                self.num_yields += 1
                                yield {
                                    'name': f'outbound_routing_#eni{eni}nsg{nsg_index}acl{acl_index}ip{ip_index}p{prefix_index}',
                                    'op': 'create',
                                    'type': 'SAI_OBJECT_TYPE_OUTBOUND_ROUTING_ENTRY',
                                    'key': {
                                        'switch_id': '$SWITCH_ID',
                                        'eni_id': f'$eni_#{eni}',
                                        'destination': f'{ip_prefix}/{mask}'
                                    },
                                    'attributes': [
                                        'SAI_OUTBOUND_ROUTING_ENTRY_ATTR_ACTION', 'SAI_OUTBOUND_ROUTING_ENTRY_ACTION_ROUTE_VNET',
                                        'SAI_OUTBOUND_ROUTING_ENTRY_ATTR_DST_VNET_ID', f'$vnet_#eni{eni}'
                                    ]
                                }
                                added_route_count += 1
                            else:
                                # routes that do not have a mac mapping
                                self.num_yields += 1
                                yield {
                                    'name': f'outbound_routing_#eni{eni}nsg{nsg_index}acl{acl_index}ip{ip_index}p{prefix_index}',
                                    'op': 'create',
                                    'type': 'SAI_OBJECT_TYPE_OUTBOUND_ROUTING_ENTRY',
                                    'key': {
                                        'switch_id': '$SWITCH_ID',
                                        'eni_id': f'$eni_#{eni}',
                                        'destination': f'{ip_prefix}/{mask}'
                                    },
                                    'attributes': [
                                        'SAI_OUTBOUND_ROUTING_ENTRY_ATTR_ACTION', 'SAI_OUTBOUND_ROUTING_ENTRY_ACTION_ROUTE_VNET_DIRECT',
                                        'SAI_OUTBOUND_ROUTING_ENTRY_ATTR_DST_VNET_ID', f'$vnet_#eni{eni}',
                                        'SAI_OUTBOUND_ROUTING_ENTRY_ATTR_OVERLAY_IP', vtep_eni
                                    ]
                                }
                                added_route_count += 1
                            ip_prefix = str(ipa(ip_prefix) + int(ipa(p.IP_STEP1)) * nr_of_ips)
                            ip_map_count += int(math.pow(2, (32 - mask)))
                    # TODO: transition between mapped and routed ips

            # TODO: write condition check here to add a default route if no route was added so curent ENI'
            if added_route_count == 0:
                remote_ip_prefix = str(ipa(p.IP_R_START) + eni_index * int(ipa(p.IP_STEP_ENI)))
                self.num_yields += 1
                yield {
                    'name': f'outbound_routing_#eni{eni}',
                    'op': 'create',
                    'type': 'SAI_OBJECT_TYPE_OUTBOUND_ROUTING_ENTRY',
                    'key': {
                        'switch_id': '$SWITCH_ID',
                        'eni_id': f'$eni_#{eni}',
                        'destination': f'{remote_ip_prefix}/10'
                    },
                    'attributes': [
                        'SAI_OUTBOUND_ROUTING_ENTRY_ATTR_ACTION', 'SAI_OUTBOUND_ROUTING_ENTRY_ACTION_ROUTE_VNET',
                        'SAI_OUTBOUND_ROUTING_ENTRY_ATTR_DST_VNET_ID', f'$vnet_#eni{eni}'
                    ]
                }


if __name__ == '__main__':
    conf = OutboundRouting()
    common_main(conf)



#!/usr/bin/python3
"""SAI generator for Address Maps"""

import os
import sys

from dpugen.confbase import (
    ConfBase,
    maca
)
from dpugen.confutils import common_main


class Mappings(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):

            eni_mac = str(
                maca(
                    int(maca(p.MAC_L_START)) + eni_index * int(maca(p.ENI_MAC_STEP))
                )
            ).replace('-', ':')

            self.num_yields += 1
            yield {
                'name': f'eni_ether_address_map_#eni{eni}',
                'op': 'create',
                'type': 'SAI_OBJECT_TYPE_ENI_ETHER_ADDRESS_MAP_ENTRY',
                'key': {
                    'switch_id': '$SWITCH_ID',
                    'address': eni_mac
                },
                'attributes': [
                    'SAI_ENI_ETHER_ADDRESS_MAP_ENTRY_ATTR_ENI_ID', f'$eni_#{eni}',
                ]
            }


if __name__ == '__main__':
    conf = Mappings()
    common_main(conf)



#!/usr/bin/python3

import os
import sys

from dpugen.confbase import (
    ConfBase,
    ipa,
    maca
)
from dpugen.confutils import common_main


class Enis(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):
            local_mac = str(maca(int(cp.MAC_L_START)+eni_index*int(maca(p.ENI_MAC_STEP)))).replace('-', ':')
            vm_underlay_dip = str(ipa(p.PAL) + eni_index * int(ipa(p.IP_STEP1)))

            acl_nsgs_in = []
            acl_nsgs_out = []

            for nsg_index in range(1, (p.ACL_NSG_COUNT*2+1)):
                nsg_id = eni_index * 1000 + nsg_index

                stage = (nsg_index - 1) % 3 + 1
                if nsg_index < 4:
                    acl_nsgs_in.append(
                        {
                            'acl-group-id': 'acl-group-%d' % nsg_id,
                            'stage': stage
                        }
                    )
                else:
                    acl_nsgs_out.append(
                        {
                            'acl-group-id': 'acl-group-%d' % nsg_id,
                            'stage': stage
                        }
                    )

            self.num_yields += 1
            yield {
                'DASH_ENI_NSG:eni-%d' % eni: {
                    'eni_id': 'eni-%d' % eni,
                    'mac_address': local_mac,
                    'underlay_ip': vm_underlay_dip,
                    'admin_state': 'enabled',
                    'vnet': 'vnet-%d' % eni,
                },
                'OP': 'SET'
            }


if __name__ == '__main__':
    conf = Enis()
    common_main(conf)



#!/usr/bin/python3

import math
import os
import sys

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class RouteTables(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params
        nr_of_routes_prefixes = int(math.log(p.IP_ROUTE_DIVIDER_PER_ACL_RULE, 2))
        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT)):
            added_route_count = 0
            for table_index in range(1, (p.ACL_NSG_COUNT*2+1)):
                for acl_index in range(1, (p.ACL_RULES_NSG+1)):
                    ip_map_count = 0
                    remote_ip = cp.IP_R_START + eni_index * cp.IP_STEP_ENI + (table_index - 1) * cp.IP_STEP_NSG + (acl_index - 1) * cp.IP_STEP_ACL
                    no_of_route_groups = p.IP_PER_ACL_RULE * 2 // p.IP_ROUTE_DIVIDER_PER_ACL_RULE
                    for ip_index in range(0, no_of_route_groups):
                        ip_prefix = remote_ip - 1 + ip_index * p.IP_ROUTE_DIVIDER_PER_ACL_RULE * cp.IP_STEP1
                        for prefix_index in range(nr_of_routes_prefixes, 0, -1):
                            # nr_of_ips = int(math.pow(2, prefix_index-1))
                            nr_of_ips = 1 << (prefix_index-1)
                            mask = 32 - prefix_index + 1
                            if mask == 32:
                                ip_prefix = ip_prefix + 1

                            if (eni % 4) == 1:
                                self.num_yields += 1
                                yield {
                                    "DASH_ROUTE_TABLE:eni-%d:%s/%d" % (eni, ip_prefix, mask): {
                                        "action_type": "vnet",
                                        "vnet": "vnet-%d" % (eni + p.ENI_L2R_STEP)
                                    },
                                    "OP": "SET"
                                }

                            else:
                                # routes that do not have a mac mapping
                                self.num_yields += 1
                                yield {
                                    "DASH_ROUTE_TABLE:eni-%d:%s/%d" % (eni, ip_prefix, mask): {
                                        "action_type": "vnet",
                                        "vnet": "vnet-%d" % (eni + p.ENI_L2R_STEP)
                                    },
                                    "OP": "SET"
                                }

                            added_route_count += 1
                            ip_prefix = ip_prefix + cp.IP_STEP1 * nr_of_ips
                            ip_map_count += int(math.pow(2, (32 - mask)))

                    # TODO1: transition between mapped and routed ips

            # TODO: write condition check here to add a default route if no route was added so curent ENI'
            if added_route_count == 0:
                remote_ip_prefix = cp.IP_R_START + eni_index * cp.IP_STEP_ENI
                self.numYields += 1
                yield {
                    "DASH_ROUTE_TABLE:eni-%d:%s/%d" % (eni, remote_ip_prefix, 10): {
                        "action_type": "vnet",
                        "vnet": "vnet-%d" % (eni + p.ENI_L2R_STEP)
                    },
                    "OP": "SET"
                }
            # route_layers = deepcopy(
            #     {
            #         "Name": "LAYER_%d_ROUTE" % eni, "Type": 1,
            #         "Groups": [
            #             {
            #                 "Name": "GROUP_%d_OUT_ROUTE" % eni,
            #                 "Type": 0, "Direction": 0,
            #                 "Rules": {
            #                     "subnet_routes": routes
            #                 }
            #             },
            #             {
            #                 "Name": "GROUP_%d_IN_ROUTE" % eni,
            #                 "Type": 0,
            #                 "Direction": 1,
            #                 "Rules": [
            #                     {
            #                         "Name": "RULE_900%d_IN_ROUTE" % eni,
            #                         "Priority": 1,
            #                         "Action": 2,
            #                         "Conditions": [
            #                             {
            #                                 "VNIKeyList": [eni, (eni + ENI_L2R_STEP)]
            #                             }
            #                         ]
            #                     }
            #                 ]
            #             }
            #         ]
            #     }
            # )


if __name__ == "__main__":
    conf = RouteTables()
    common_main(conf)



#!/usr/bin/python3

import io
import os
import sys

from dpugen.confbase import (
    ConfBase,
    maca
)
from dpugen.confutils import common_main


class Mappings(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT)):
            debug_file = io.open('macs_for_eni_%d.txt' % eni, "wt")
            # vtep_local = cp.PAL + eni_index * cp.IP_STEP1
            vtep_remote = cp.PAR + eni_index * cp.IP_STEP1

            r_vni_id = eni + p.ENI_L2R_STEP

            if (eni % 4) == 1:
                # mapped IPs

                for nsg_index in range(1, (p.ACL_NSG_COUNT*2+1)):
                    for ip_index in range(1, (p.ACL_RULES_NSG//2+1)):
                        remote_ip_a = cp.IP_R_START + eni_index * cp.IP_STEP_ENI + (nsg_index - 1) * cp.IP_STEP_NSG + (ip_index - 1) * cp.IP_STEP_ACL
                        remote_mac_a = str(
                            maca(
                                int(maca(p.MAC_R_START)) +
                                eni_index * int(maca(p.ENI_MAC_STEP)) +
                                (nsg_index - 1) * int(maca(p.ACL_NSG_MAC_STEP)) +
                                (ip_index - 1) * int(maca(p.ACL_POLICY_MAC_STEP))
                            )
                        ).replace('-', ':')

                        debug_file.write('############ ALLOW\n')
                        for i in range(p.IP_MAPPED_PER_ACL_RULE):
                            remote_expanded_ip = remote_ip_a + i * 2
                            remote_expanded_mac = str(
                                maca(
                                    int(maca(remote_mac_a)) + i * 2
                                )
                            ).replace('-', ':')

                            self.num_yields += 1
                            yield {
                                "DASH_VNET_MAPPING_TABLE:vnet-%d:%s" % (r_vni_id, remote_expanded_ip): {
                                    "routing_type": "vnet_encap",
                                    "underlay_ip": str(vtep_remote),
                                    "mac_address": remote_expanded_mac,
                                    "use_dst_vni": "true"
                                },
                                "OP": "SET"
                            }
                            debug_file.write(remote_expanded_mac + '\n')

                        remote_ip_d = remote_ip_a - 1
                        debug_file.write('############ DENNY\n')
                        for i in range(p.IP_MAPPED_PER_ACL_RULE):
                            remote_expanded_ip = remote_ip_d + i * 2
                            remote_expanded_mac = str(
                                maca(
                                    int(maca(remote_mac_a)) + i * 2 - 1
                                )
                            ).replace('-', ':')

                            self.num_yields += 1
                            yield {
                                "DASH_VNET_MAPPING_TABLE:vnet-%d:%s" % (r_vni_id, remote_expanded_ip): {
                                    "routing_type": "vnet_encap",
                                    "underlay_ip": str(vtep_remote),
                                    "mac_address": remote_expanded_mac,
                                    "use_dst_vni": "true"
                                },
                                "OP": "SET"
                            }
                            debug_file.write(remote_expanded_mac + '\n')
            else:
                # routed IPs
                remote_expanded_ip = cp.IP_R_START + eni_index * cp.IP_STEP_ENI

                remote_expanded_mac = str(
                    maca(
                        int(maca(p.MAC_R_START)) +
                        eni_index * int(maca(p.ENI_MAC_STEP))
                    )
                ).replace('-', ':')

                self.num_yields += 1
                yield {
                    "DASH_VNET_MAPPING_TABLE:vnet-%d:%s" % (r_vni_id, remote_expanded_ip): {
                        "routing_type": "vnet_encap",
                        "underlay_ip": str(vtep_remote),
                        "mac_address": remote_expanded_mac,
                        "use_dst_vni": "true"
                    },
                    "OP": "SET"
                }
                debug_file.write(remote_expanded_mac + '\n')
            debug_file.close()


if __name__ == '__main__':
    conf = Mappings()
    common_main(conf)



#!/usr/bin/python3

import os
import sys

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class RouteRules(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT)):
            r_vni_id = eni + p.ENI_L2R_STEP
            vtep_remote = cp.PAR + eni_index * cp.IP_STEP1
            self.num_yields += 1
            yield {
                "DASH_ROUTE_RULE_TABLE:eni-%d:%d:%s/32" % (eni, r_vni_id, vtep_remote): {
                    "action_type": "decap",
                    "priority": "1",
                    # "protocol": "0",
                    "pa_validation": "true",
                    "vnet":  "vnet-%d" % r_vni_id
                },
                "OP": "SET"
            }


'''
            self.num_yields += 1
            yield {
                "DASH_ROUTE_RULE_TABLE:eni-%d:%d:10.0.2.0/24" % (eni, r_vpc): {
                    "action_type": "decap",
                    "priority": "1",
                    "protocol": "0",
                    "pa_validation": "false"
                },
                "OP": "SET"
            }
'''

if __name__ == "__main__":
    conf = RouteRules()
    common_main(conf)



#!/usr/bin/python3

import os
import sys

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class PrefixTags(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params
        for eni_index in range(1, p.ENI_COUNT+1):
            IP_L = cp.IP_L_START + (eni_index - 1) * cp.IP_STEP_ENI
            r_vpc = eni_index + p.ENI_L2R_STEP
            IP_R = cp.IP_R_START + (eni_index - 1) * cp.IP_STEP_ENI
            self.num_yields += 1
            yield \
                {
                    "PREFIX-TAG:VPC:%d" % eni_index: {
                        "prefix-tag-id": "%d" % eni_index,
                        "prefix-tag-number": eni_index,
                        "ip-prefixes-ipv4": [
                            "%s/32" % IP_L
                        ]
                    },
                }

            self.num_yields += 1
            yield \
                {
                    "PREFIX-TAG:VPC:%d" % r_vpc: {
                        "prefix-tag-id": "%d" % r_vpc,
                        "prefix-tag-number": r_vpc,
                        "ip-prefixes-ipv4": [
                            "%s/9" % IP_R
                        ]
                    },
                }


if __name__ == "__main__":
    conf = PrefixTags()
    common_main(conf)



#!/usr/bin/python3
"""SAI generator for Acl Groups"""

import os
import sys

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class AclGroups(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):
            for nsg_index in range(1, (p.ACL_NSG_COUNT+1)):

                self.num_yields += 1
                yield {
                    'name': f'in_acl_group_#eni{eni}nsg{nsg_index}',
                    'op': 'create',
                    'type': 'SAI_OBJECT_TYPE_DASH_ACL_GROUP',
                    'attributes': [
                        'SAI_DASH_ACL_GROUP_ATTR_IP_ADDR_FAMILY', 'SAI_IP_ADDR_FAMILY_IPV4',
                    ]
                }

                self.num_yields += 1
                yield {
                    'name': f'out_acl_group_#eni{eni}nsg{nsg_index}',
                    'op': 'create',
                    'type': 'SAI_OBJECT_TYPE_DASH_ACL_GROUP',
                    'attributes': [
                        'SAI_DASH_ACL_GROUP_ATTR_IP_ADDR_FAMILY', 'SAI_IP_ADDR_FAMILY_IPV4',
                    ]
                }


if __name__ == '__main__':
    conf = AclGroups()
    common_main(conf)



#!/usr/bin/python3
"""SAI generator for ENI"""

import os
import sys

from dpugen.confbase import (
    ConfBase,
    ipa
)
from dpugen.confutils import common_main


class Enis(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):
            vm_underlay_dip = str(ipa(p.PAL) + eni_index * int(ipa(p.IP_STEP1)))

            eni_data = {
                'name': f'eni_#{eni}',
                'op': 'create',
                'type': 'SAI_OBJECT_TYPE_ENI',
                'attributes': [
                    'SAI_ENI_ATTR_CPS', '10000',
                    'SAI_ENI_ATTR_PPS', '100000',
                    'SAI_ENI_ATTR_FLOWS', '100000',
                    'SAI_ENI_ATTR_ADMIN_STATE', 'True',
                    'SAI_ENI_ATTR_VM_UNDERLAY_DIP', vm_underlay_dip,
                    'SAI_ENI_ATTR_VM_VNI', f'{eni}',
                    'SAI_ENI_ATTR_VNET_ID', f'$vnet_#eni{eni}',
                    'SAI_ENI_ATTR_INBOUND_V4_STAGE1_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V4_STAGE2_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V4_STAGE3_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V4_STAGE4_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V4_STAGE5_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V6_STAGE1_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V6_STAGE2_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V6_STAGE3_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V6_STAGE4_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_INBOUND_V6_STAGE5_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V4_STAGE1_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V4_STAGE2_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V4_STAGE3_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V4_STAGE4_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V4_STAGE5_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V6_STAGE1_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V6_STAGE2_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V6_STAGE3_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V6_STAGE4_DASH_ACL_GROUP_ID', '0',
                    'SAI_ENI_ATTR_OUTBOUND_V6_STAGE5_DASH_ACL_GROUP_ID', '0',
                ]
            }
            for nsg_index in range(1, (p.ACL_NSG_COUNT + 1)):
                stage_index = eni_data['attributes'].index(
                    f'SAI_ENI_ATTR_INBOUND_V4_STAGE{nsg_index}_DASH_ACL_GROUP_ID')
                eni_data['attributes'][stage_index + 1] = f'$in_acl_group_#eni{eni}nsg{nsg_index}'
                stage_index = eni_data['attributes'].index(
                    f'SAI_ENI_ATTR_OUTBOUND_V4_STAGE{nsg_index}_DASH_ACL_GROUP_ID')
                eni_data['attributes'][stage_index + 1] = f'$out_acl_group_#eni{eni}nsg{nsg_index}'

            self.num_yields += 1
            yield eni_data


if __name__ == '__main__':
    conf = Enis()
    common_main(conf)



#!/usr/bin/python3
"""SAI generator for Outbound CA PA"""

import os
import sys

from dpugen.confbase import (
    ConfBase,
    ipa,
    maca
)
from dpugen.confutils import common_main


class OutboundCaToPa(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT * p.ENI_STEP, p.ENI_STEP)):

            vtep_eni = str(ipa(p.PAL) + int(ipa(p.IP_STEP1)) * eni_index)
            vtep_remote = str(ipa(p.PAR) + int(ipa(p.IP_STEP1)) * eni_index)

            # 1 in 4 enis will have all its ips mapped
            if (eni % 4) == 1:
                print(f'    mapped:eni:{eni}', file=sys.stderr)
                for nsg_index in range(1, (p.ACL_NSG_COUNT*2+1)):
                    for acl_index in range(1, (p.ACL_RULES_NSG//2+1)):
                        remote_ip_a = ipa(p.IP_R_START) + eni_index * int(ipa(p.IP_STEP_ENI)) + \
                            (nsg_index - 1) * int(ipa(p.IP_STEP_NSG)) + (acl_index - 1) * int(ipa(p.IP_STEP_ACL))
                        remote_mac_a = str(
                            maca(
                                int(maca(p.MAC_R_START)) +
                                eni_index * int(maca(p.ENI_MAC_STEP)) +
                                (nsg_index - 1) * int(maca(p.ACL_NSG_MAC_STEP)) +
                                (acl_index - 1) * int(maca(p.ACL_POLICY_MAC_STEP))
                            )
                        ).replace('-', ':')

                        # mapping for allow ip
                        for i in range(p.IP_MAPPED_PER_ACL_RULE):
                            remote_expanded_ip = str(remote_ip_a + i * 2)
                            remote_expanded_mac = str(
                                maca(
                                    int(maca(remote_mac_a)) + i * 2
                                )
                            ).replace('-', ':')

                            self.num_yields += 1
                            yield {
                                'name': f'outbound_ca_to_pa_#eni{eni}nsg{nsg_index}acl{acl_index}i{i}A',
                                'op': 'create',
                                'type': 'SAI_OBJECT_TYPE_OUTBOUND_CA_TO_PA_ENTRY',
                                'key': {
                                    'switch_id': '$SWITCH_ID',
                                    'dst_vnet_id': f'$vnet_#eni{eni}',
                                    'dip': remote_expanded_ip
                                },
                                'attributes': [
                                    'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_UNDERLAY_DIP', vtep_remote,
                                    'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_OVERLAY_DMAC', remote_expanded_mac,
                                    'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_USE_DST_VNET_VNI', 'True'
                                ]
                            }

                        # mapping for deny ip
                        # if you want to test that packets are being dropped because of the acl rule keep it
                        # if you want to test that packets are being dropped because no mapping you can comment this area
                        for i in range(p.IP_MAPPED_PER_ACL_RULE):
                            remote_expanded_ip = str(remote_ip_a + i * 2 - 1)
                            remote_expanded_mac = str(
                                maca(
                                    int(maca(remote_mac_a)) + i * 2 - 1
                                )
                            ).replace('-', ':')

                            self.num_yields += 1
                            yield {
                                'name': f'outbound_ca_to_pa_#eni{eni}nsg{nsg_index}acl{acl_index}i{i}D',
                                'op': 'create',
                                'type': 'SAI_OBJECT_TYPE_OUTBOUND_CA_TO_PA_ENTRY',
                                'key': {
                                    'switch_id': '$SWITCH_ID',
                                    'dst_vnet_id': f'$vnet_#eni{eni}',
                                    'dip': remote_expanded_ip
                                },
                                'attributes': [
                                    'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_UNDERLAY_DIP', vtep_remote,
                                    'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_OVERLAY_DMAC', remote_expanded_mac,
                                    'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_USE_DST_VNET_VNI', 'True'
                                ]
                            }

            # 3 in 4 enis will have just mapping for gateway ip, for ip that are only routed and not mapped
            else:
                print(f'    routed:eni:{eni}', file=sys.stderr)

                remote_expanded_mac = str(
                    maca(
                        int(maca(p.MAC_R_START)) +
                        eni_index * int(maca(p.ENI_MAC_STEP))
                    )
                ).replace('-', ':')

                self.num_yields += 1
                yield {
                    'name': f'outbound_ca_to_pa_#eni{eni}',
                    'op': 'create',
                    'type': 'SAI_OBJECT_TYPE_OUTBOUND_CA_TO_PA_ENTRY',
                    'key': {
                        'switch_id': '$SWITCH_ID',
                        'dst_vnet_id': f'$vnet_#eni{eni}',
                        'dip': vtep_eni
                    },
                    'attributes': [
                        'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_UNDERLAY_DIP', vtep_remote,
                        'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_OVERLAY_DMAC', remote_expanded_mac,
                        'SAI_OUTBOUND_CA_TO_PA_ENTRY_ATTR_USE_DST_VNET_VNI', 'True'
                    ]
                }


if __name__ == '__main__':
    conf = OutboundCaToPa()
    common_main(conf)



#!/usr/bin/python3

import os
import sys

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class Vpcs(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params
        IP_L_START = cp.IP_L_START
        IP_R_START = cp.IP_R_START
        IP_STEP_ENI = cp.IP_STEP_ENI
        ENI_L2R_STEP = p.ENI_L2R_STEP
        ENI_COUNT = p.ENI_COUNT

        for eni_index in range(1, ENI_COUNT+1):
            IP_L = IP_L_START + (eni_index - 1) * IP_STEP_ENI
            r_vpc = eni_index + ENI_L2R_STEP
            IP_R = IP_R_START + (eni_index - 1) * IP_STEP_ENI
            self.num_yields += 1
            yield {
                "VPC:%d" % eni_index: {
                    "vpc-id": "vpc-%d" % eni_index,
                    "vni-key": eni_index,
                    "encap": "vxlan",
                    "address_spaces": [
                        "%s/32" % IP_L
                    ]
                },
            }

            self.num_yields += 1
            yield {
                "VPC:%d" % r_vpc: {
                    "vpc-id": "vpc-%d" % r_vpc,
                    "vni-key": r_vpc,
                    "encap": "vxlan",
                    "address_spaces": [
                        "%s/9" % IP_R
                    ]
                },
            }


if __name__ == "__main__":
    conf = Vpcs()
    common_main(conf)



#!/usr/bin/python3
"""SAI generator for VIP"""

import os
import sys

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class Vips(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        self.num_yields += 1
        yield {
            'name': 'vip_#1',
            'op': 'create',
            'type': 'SAI_OBJECT_TYPE_VIP_ENTRY',
            'key': {
                'switch_id': '$SWITCH_ID',
                'vip': p.LOOPBACK
            },
            'attributes': [
                'SAI_VIP_ENTRY_ATTR_ACTION', 'SAI_VIP_ENTRY_ACTION_ACCEPT',
            ]
        }


if __name__ == '__main__':
    conf = Vips()
    common_main(conf)



#!/usr/bin/python3

import os
import sys

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class VpcMappingTypes(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)

        vpcmappingtypes = [
            "vpc",
            "privatelink",
            "privatelinknsg"
        ]

        # return generator from list for consistency with other subgenerators
        for x in vpcmappingtypes:

            self.num_yields += 1
            yield x


if __name__ == "__main__":
    conf = VpcMappingTypes()
    common_main(conf)



#!/usr/bin/python3

import os
import sys

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class Vnets(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params

        for eni_index, eni in enumerate(range(p.ENI_START, p.ENI_START + p.ENI_COUNT)):
            self.num_yields += 1

            yield {
                'DASH_VNET_TABLE:vnet-%d' % eni: {
                    'vni': eni,
                },
                'OP': 'SET'
            }

            self.num_yields += 1
            r_vni_id = p.ENI_L2R_STEP + eni

            yield {
                'DASH_VNET_TABLE:vnet-%d' % r_vni_id: {
                    'vni': r_vni_id,
                },
                'OP': 'SET'
            }


if __name__ == '__main__':
    conf = Vnets()
    common_main(conf)



#!/usr/bin/python3

import os
import sys
from copy import deepcopy

from dpugen.confbase import ConfBase
from dpugen.confutils import common_main


class AclGroups(ConfBase):

    def __init__(self, params={}):
        super().__init__(params)
        self.num_yields = 0

    def items(self):
        print('  Generating %s ...' % os.path.basename(__file__), file=sys.stderr)
        p = self.params
        cp = self.cooked_params
        IP_STEP1 = cp.IP_STEP1
        IP_STEP_ACL = cp.IP_STEP_ACL
        IP_STEP_NSG = cp.IP_STEP_NSG
        IP_STEP_ENI = cp.IP_STEP_ENI
        IP_STEPE = cp.IP_STEPE
        IP_R_START = cp.IP_R_START
        IP_L_START = cp.IP_L_START
        ACL_NSG_COUNT = p.ACL_NSG_COUNT
        ACL_RULES_NSG = p.ACL_RULES_NSG
        IP_PER_ACL_RULE = p.IP_PER_ACL_RULE

        for eni_index in range(1, p.ENI_COUNT + 1):
            local_ip = IP_L_START + (eni_index - 1) * IP_STEP_ENI
            l_ip_ac = deepcopy(str(local_ip)+"/32")

            for table_index in range(1, (ACL_NSG_COUNT*2+1)):
                table_id = eni_index * 1000 + table_index

                rules = []
                rappend = rules.append
                for ip_index in range(1, (ACL_RULES_NSG+1), 2):
                    remote_ip_a = IP_R_START + (eni_index - 1) * IP_STEP_ENI + (
                        table_index - 1) * IP_STEP_NSG + (ip_index - 1) * IP_STEP_ACL

                    ip_list_a = [str(remote_ip_a + expanded_index * IP_STEPE) +
                                 "/32" for expanded_index in range(0, IP_PER_ACL_RULE)]
                    ip_list_a.append(l_ip_ac)

                    rule_a = {
                        "priority": ip_index,
                        "action": "allow",
                        "terminating": False,
                        "src_addrs": ip_list_a[:],
                        "dst_addrs":  ip_list_a[:],
                    }
                    rappend(rule_a)
                    remote_ip_d = remote_ip_a + IP_STEP1

                    ip_list_d = [str(remote_ip_d + expanded_index * IP_STEPE) +
                                 "/32" for expanded_index in range(0, IP_PER_ACL_RULE)]
                    ip_list_d.append(l_ip_ac)

                    rule_d = {
                        "priority": ip_index+1,
                        "action": "deny",
                        "terminating": True,
                        "src_addrs": ip_list_d[:],
                        "dst_addrs":  ip_list_d[:],
                    }
                    rappend(rule_d)

                # add as last rule in last table from ingress and egress an allow rule for all the ip's from egress and ingress
                if ((table_index - 1) % 3) == 2:
                    all_ipsA = IP_R_START + (eni_index - 1) * IP_STEP_ENI + (table_index % 6) * IP_STEP_NSG
                    all_ipsB = all_ipsA + 1 * IP_STEP_NSG
                    all_ipsC = all_ipsA + 2 * IP_STEP_NSG

                    ip_list_all = [
                        l_ip_ac,
                        str(all_ipsA)+"/14",
                        str(all_ipsB)+"/14",
                        str(all_ipsC)+"/14",
                    ]

                    rule_allow_all = {
                        "priority": ip_index+2,
                        "action": "allow",
                        "terminating": "true",
                        "src_addrs": ip_list_all[:],
                        "dst_addrs":  ip_list_all[:],
                    }
                    rappend(rule_allow_all)

                acl_group = deepcopy(
                    {
                        "ACL-GROUP:ENI:%d:TABLE:%d" % (eni_index, table_id): {
                            "acl-group-id": "acl-group-%d" % table_id,
                            "ip_version": "IPv4",
                            "rules": rules
                        }
                    }
                )
                self.num_yields += 1
                yield acl_group


if __name__ == "__main__":
    conf = AclGroups()
    common_main(conf)
